#!/usr/bin/python

import subprocess
import os
import sys
import shutil
import xml.dom.minidom
import xml.dom

WORKING_DIR = os.getcwd()

os.chdir(os.getcwd() + "/../..")

PROJECT_DIR = os.getcwd() + "/"
SCRIPT_DEPLOYMENT_DIR = PROJECT_DIR + "res/scripts/sys/"
MSI_FRAGMENT_FILE = PROJECT_DIR + "build/msi/source/mysql_workbench_fragment.xml"
CMAKE_FILE = PROJECT_DIR + "res/scripts/CMakeLists.txt"

print "Working dir  : " + WORKING_DIR
print "Project dir  : " + PROJECT_DIR
print "MSI Frag file: " + MSI_FRAGMENT_FILE
print "CMake file   : " + CMAKE_FILE


# Class to process sp_helper* files to generate wbsys files.
# A sp_helper script runs other script files to do it's task. In
# WB, we need a single file with all the scripts.
class SQLFileProcessor:

    # Process a sp_helper file in order to generate a wbsys file
    def process_file(self, source):
        print "  - Processing " + source + " file"
    
        source_prefix = "ps_helper_"
        dest_prefix = "wbsys_"
    
        dest_filename = SCRIPT_DEPLOYMENT_DIR + dest_prefix + source.replace(source_prefix, "")
    
        # Truncate file
        dest_file = open(dest_filename, "w")
        dest_file.close()
    
        with open(source) as f:
            for line in f:
                line = line.strip()
        
                if not line.lower().startswith("source"):
                    continue
        
                line = line.replace("SOURCE", "")
                line = line.replace("source", "")
                line = line.replace("./", "")
                line = line.strip()
        
                self.append_file(line, dest_filename)        
        
        print "  - WBSYS file generated at  " + dest_filename

    # Append the contents of the 'source' file to the 'dest' file
    def append_file(self, source, dest):

        print "    - Appending " + source + " to " + dest

        dest_handle = open(dest, "a")

        print >> dest_handle, "/* "
        print >> dest_handle, "   ==================================================================="
        print >> dest_handle, "   Contents from file: " + source
        print >> dest_handle, "   ==================================================================="
        print >> dest_handle, "*/"
        print >> dest_handle, "  "
        print >> dest_handle, "  "

        with open(source) as f:
            for line in f:
                line = line.replace("ps_helper", "wbsys")
                line = line.replace("\n", "")
                print >> dest_handle, line




# This class searches a MSI XML fragment file for the wbsys 
# files, so that they get installed along with WB. The fragment
# file is updated with the processed wbsys files
class XMLFileManager:
  
    # The constructor reads the fragment file and finds the right
    # component element that will contain the wbsys files
    def __init__(self):
        self.xml_document = xml.dom.minidom.parse(MSI_FRAGMENT_FILE)
    
        self.root = self.xml_document.documentElement
        directory_elements = self.root.getElementsByTagName("Directory")
  
        for directory_element in directory_elements:
    
            if directory_element.getAttribute("Id") == "wbsys_scripts_dir":
                self.component_element = directory_element.getElementsByTagName("Component").item(0)
                print "Found the right tag!!! Name:" + directory_element.getAttribute("Name")
                break
    
        
    # Overwrites the original XML file
    def write_file(self):
        indent = self.component_element.firstChild
        lastIdent = self.component_element.lastChild
    
        while self.component_element.firstChild != None:
            self.component_element.removeChild(self.component_element.firstChild)
                
        index = 0
        for filename in os.listdir(SCRIPT_DEPLOYMENT_DIR):
            index += 1
            
            newEntry = self.xml_document.createElement("File")
            newEntry.setAttribute("Id", "wbsys" + str(index))
            newEntry.setAttribute("Name", filename)

            self.component_element.appendChild(indent.cloneNode(False))
            self.component_element.appendChild(newEntry)
        
        self.component_element.appendChild(lastIdent.cloneNode(False))
        
        f = open(MSI_FRAGMENT_FILE, mode='w')
        self.xml_document.writexml(f)
        f.close()
           



# This class deals with a CMakeLists.txt file in order
# to setup the proper wbsys files to install.
class CMakeFileManager:
    file_lines_before = None
    file_lines = None
    file_lines_after = None
    state = 0
    
    # The constructor read the file into line arrays, so
    # They can be writen again properly. The automates files
    # are always discarted, and new ones are set during the
    # process.
    def __init__(self):
        with open(CMAKE_FILE) as f:
            self.file_lines_before = []
            self.file_lines_after = []
            self.file_lines = []
            
            for line in f:
                if self.state == 0:
                    self.file_lines_before.append(line)
                    if line.strip().find("set(WBSYS_FILES") > -1:
                        self.state = 1
                elif self.state == 1:
                    if line.strip().find(")") > -1:
                        self.file_lines_after.append(line)
                        self.state = 2
                else:
                    self.file_lines_after.append(line)

    def write_file(self):
        # Get the files to insert
        for filename in os.listdir(SCRIPT_DEPLOYMENT_DIR):
            self.file_lines.append("    sys/" + os.path.basename(filename) + "\n")        
        
        # Overwrite the original file
        f = open(CMAKE_FILE, mode='w')
        f.writelines(self.file_lines_before)
        f.writelines(self.file_lines)
        f.writelines(self.file_lines_after)
        f.close()
        







# TODO: Remove this when done
os.chdir(WORKING_DIR)
print "Preparing...please be patient..."
print "  - Checking for old paths..."
if (os.path.isdir("dbahelper")):
    print "    - Removing existing dbahelper directory"
    shutil.rmtree("dbahelper")

if (os.path.isdir("wbsys")):
    print "    - Removing existing wbsys directory"
    shutil.rmtree("wbsys")


print "  - Checking git proxy configuration..."
proc = subprocess.Popen(["git", "config", "--global", "http.proxy"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

out, err = proc.communicate()

if not out.strip():
    print "    - WARNING: No proxy configured. Try using git config --global http.proxy <proxy>"
else:
    print "    - Using proxy: " + out.strip()



print "  - Cloning remote repository..."  
proc = subprocess.Popen(["git", "clone", "https://github.com/MarkLeith/dbahelper.git"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

out, err = proc.communicate()

if proc.returncode > 0:
    print "    - ERROR[" + str(proc.returncode) + "] - There was an error on 'git clone'"
    print err
    sys.exit(1)






print "Processing files..."

os.chdir("dbahelper")
entries = os.listdir(os.curdir)

sql_processor = SQLFileProcessor()
xml_manager = XMLFileManager()
cmake_manager = CMakeFileManager()


# This would print all the files and directories
for entry in entries:
    if os.path.isdir(entry):
        continue
  
    if entry.startswith("ps_helper") and entry.endswith(".sql"):
        sql_processor.process_file(entry)


# The destructors will take care of writing the files
cmake_manager.write_file()
xml_manager.write_file()

# Cleanup after using it
os.chdir(WORKING_DIR)
if (os.path.isdir("dbahelper")):
    shutil.rmtree("dbahelper")

